AUTH = '/auth'
SESSIONS = '/sessions'
USERS = '/users'
PODS = '/pods'
UNITS = '/units'
CHARGE_SCHEDULES = '/charge-schedules'
CHARGES = '/charges'

API_BASE = 'api.pod-point.com'
API_VERSION = 'v4'
API_BASE_URL = 'https://' + API_BASE + '/' + API_VERSION


from argparse import _AttributeHolder
import logging

import aiohttp
from ..errors import SessionError
from ..endpoints import API_BASE_URL, SESSIONS
from .helpers import Helpers, APIWrapper

_LOGGER: logging.Logger = logging.getLogger(__package__)

class Session:
    def __init__(self, email: str, password: str, access_token: str, session: aiohttp.ClientSession) -> None:
        self.email: str = email
        self.password: str = password
        self.access_token: str = access_token
        self.session_id: str = None
        self.user_id: str = None
        self._session: aiohttp.ClientSession = session
        self._helpers: Helpers = Helpers()

    async def create(self):
        url = f"{API_BASE_URL}{SESSIONS}"
        headers = self._helpers.auth_headers(self.access_token)
        payload = {"email": self.email, "password": self.password}

        return_value = False

        try:
            wrapper = APIWrapper(session=self._session)
            response = await wrapper.post(url, body=payload, headers=headers, exception_class=SessionError)

            json = await response.json()

            if json['sessions']:
                _LOGGER.debug("Setting session")
                self.user_id = json['sessions']['user_id']
                self.session_id = json['sessions']['id']
                return_value = True
        except KeyError as exception:
            _LOGGER.error(f"Error processing session response {exception}")
            raise SessionError(exception)

        return return_value


from datetime import datetime
from dataclasses import dataclass
from re import M
from typing import Dict, Any

class Charge:
    def __init__(self, data: Dict[str, Any]):
        self.id: int             = data.get('id', None)
        self.kwh_used: float     = data.get('kwh_used', 0.0)
        self.duration: int       = data.get('duration', 0)
        self.starts_at: datetime = data.get('starts_at', None)
        self.ends_at: datetime   = data.get('ends_at', None)
        self.energy_cost: int    = data.get('energy_cost', None)

        billing_event_data = data.get('billing_event', {})
        self.billing_event = self.BillingEvent(
            id                   = billing_event_data.get('id', None),
            amount               = billing_event_data.get('amount', None),
            currency             = billing_event_data.get('currency', None),
            exchange_rate        = billing_event_data.get('exchange_rate', 0),
            presentment_amount   = billing_event_data.get('presentment_amount', None),
            presentment_currency = billing_event_data.get('presentment_currency', None)
        )

        location_data = data.get('location', {})
        self.location = self.Location(data=location_data)

        pod_data = data.get('pod', {})
        self.pod = self.Pod(id=pod_data.get('id', None))

        organisation_data = data.get('organisation', {})
        self.organisation = self.Organisation(
            id = organisation_data.get('id', None),
            name = organisation_data.get('name', None)
        )


    @dataclass
    class BillingEvent:
        id: int                   = None
        amount: Any               = None
        currency: Any             = None
        exchange_rate: int        = 0
        presentment_amount: Any   = None
        presentment_currency: Any = None


    class Location:
        def __init__(self, data: Dict[str, Any]):
            self.id       = data.get('id', None)
            self.home     = data.get('home', None)
            self.timezone = data.get('timezone', None)

            address_data = data.get('address', {})
            self.address = self.Address(
                id=address_data.get('id', None),
                business_name=address_data.get('business_name', "")
            )


        @dataclass
        class Address:
            id: int            = None
            business_name: str = ""


    @dataclass
    class Pod:
        id: int = None

    @dataclass
    class Organisation:
        id: int   = None
        name: str = None



"""Sample API Client."""
import logging
from typing import Optional, Dict, Any, List
import aiohttp
from datetime import datetime, timedelta

from podpointclient.schedule import Schedule

from .endpoints import API_BASE_URL, CHARGE_SCHEDULES, PODS, UNITS, USERS, CHARGES
from .helpers.auth import Auth
from .helpers.helpers import APIWrapper, Helpers
from .factories import PodFactory, ScheduleFactory, ChargeFactory
from .pod import Pod

TIMEOUT = 10

_LOGGER: logging.Logger = logging.getLogger(__package__)

HEADERS = {"Content-type": "application/json; charset=UTF-8"}


class PodPointClient:
    """API Client for communicating with Pod Point."""

    def __init__(
        self, username: str, password: str, session: aiohttp.ClientSession = aiohttp.ClientSession()
    ) -> None:
        """Pod Point API Client."""
        self.email = username
        self.password = password
        self._session = session
        self.auth = Auth(
            email=self.email, 
            password=self.password, 
            session=self._session
        )
        self.api_wrapper = APIWrapper(session=self._session)

    async def async_get_pods(self) -> List[Pod]:
        """Get pods from the API."""
        await self.auth.async_update_access_token()

        path = f"{USERS}/{self.auth.user_id}{PODS}"
        url = f"{API_BASE_URL}{path}"

        includes = ["statuses", "price", "model", "unit_connectors", "charge_schedules"]
        params = {"perpage": "all", "include": ",".join(includes)}

        helpers = Helpers()
        headers = helpers.auth_headers(access_token=self.auth.access_token)

        response = await self.api_wrapper.get(url=url, params=params, headers=headers)

        _LOGGER.debug(response)

        json = await response.json()

        factory = PodFactory()
        pods = factory.build_pods(pods_response=json)

        return pods

    async def async_get_pod(self, pod_id: int) -> Pod:
        """Get specific pod from the API."""
        pods = await self.async_get_pods()
        return next((pod for pod in pods if pod.id == pod_id), None)

    async def async_set_schedule(self, enabled: bool, pod: Pod) -> bool:
        """Send data from the API."""
        await self.auth.async_update_access_token()

        unit_id = pod.unit_id

        _LOGGER.info(
            f"Updating pod schedule for unit {unit_id}. Enabling schedule: {enabled}"
        )
        
        path = f"{UNITS}/{unit_id}{CHARGE_SCHEDULES}"
        url = f"{API_BASE_URL}{path}"

        helpers = Helpers()
        headers = helpers.auth_headers(access_token=self.auth.access_token)
        payload = self._schedule_data(enabled)

        _LOGGER.debug(url)
        _LOGGER.debug(headers)
        _LOGGER.debug(payload)

        response = await self.api_wrapper.put(url=url, body=payload, headers=headers)

        if response.status == 201:
            return True
        else:
            text = await response.text()
            _LOGGER.warn("Expected to recieve 201 status code when creating schedules. Got (%s) - %s", response.status, text)
            return False

    async def async_get_charges(self, home_only: bool = False, per_page: str = "5", page: str = "1"):
        """Get charges from the API."""
        await self.auth.async_update_access_token()

        path = f"{USERS}/{self.auth.user_id}{CHARGES}"
        url = f"{API_BASE_URL}{path}"
        params = {"perpage": per_page, "page": page}

        helpers = Helpers()
        headers = helpers.auth_headers(access_token=self.auth.access_token)

        response = await self.api_wrapper.get(url=url, params=params, headers=headers)

        _LOGGER.debug(response)

        json = await response.json()

        factory = ChargeFactory()
        pods = factory.build_charges(charge_response=json)

        return pods

    def _schedule_data(self, enabled: bool) -> Dict[str, Any]:
        factory = ScheduleFactory()
        schedules: List[Schedule] = factory.build_schedules(enabled=enabled)

        d_list = list(map(lambda schedule: schedule.dict, schedules))

        return {"data": d_list}



import logging
from datetime import datetime, timedelta

import aiohttp

from ..errors import AuthError, SessionError
from .session import Session
from ..endpoints import API_BASE_URL, AUTH
from .helpers import APIWrapper, HEADERS

_LOGGER: logging.Logger = logging.getLogger(__package__)

class Auth():
    """Manages authentication lifecycle for a user"""
    def __init__(self, email: str, password: str, session: aiohttp.ClientSession):
        self.email: str = email
        self.password: str = password
        self.access_token: str = None
        self.access_token_expiry: datetime = None
        self._session: aiohttp.ClientSession = session
        self._api_wrapper: APIWrapper = APIWrapper(session=self._session)

    @property
    def user_id(self):
        if self._session:
            return self._session.user_id
        return None

    def check_access_token(self) -> bool:
        """Does the current access token need refreshing?"""
        access_token_set: bool = (
            self.access_token is not None and self.access_token_expiry is not None
        )
        access_token_not_expired: bool = (
            access_token_set and datetime.now() < self.access_token_expiry
        )

        if access_token_set and access_token_not_expired:
            return True
        else:
            return False

    async def async_update_access_token(self) -> bool:
        """Update access token, if needed."""
        _LOGGER.debug(self.check_access_token())
        if self.check_access_token():
            return True

        try:
            session_created: bool = False
            _LOGGER.debug('Updating access token')
            access_token_updated: bool = await self.__update_access_token()
            _LOGGER.debug('Done')

            if access_token_updated:
                _LOGGER.debug(
                    f"Updated access token. New expiration: {self.access_token_expiry}"
                )

                self._session = Session(email=self.email, password=self.password, access_token=self.access_token, session=self._session)
                session_created = await self._session.create()

                if session_created is False:
                    _LOGGER.error("Error creating session")
            else:
                _LOGGER.error("Error updating access token")

            _LOGGER.debug("Going to return with:")
            _LOGGER.debug((access_token_updated and session_created))
            _LOGGER.debug(access_token_updated)
            _LOGGER.debug(session_created)

            return (access_token_updated and session_created)
        except AuthError as exception:
            _LOGGER.error("Error updating access token. %s", exception)
        except SessionError as exception:
            _LOGGER.error("Error creating session. %s", exception)

    async def __update_access_token(self) -> bool:
        return_value = False

        _LOGGER.info("Updating Pod Point access token")

        url = f"{API_BASE_URL}{AUTH}"
        payload = {"username": self.email, "password": self.password}

        try:
            print(url)
            print(payload)
            print(HEADERS)

            _LOGGER.debug('Posting to wrapper')
            wrapper = APIWrapper(session=self._session)
            response = await wrapper.post(url, body=payload, headers=HEADERS)

            _LOGGER.debug(response)

            if response.status != 200:
                await self.__handle_response_error(response, AuthError)
            else:
                json = await response.json()
                _LOGGER.debug(json)
                self.access_token = json["access_token"]
                self.access_token_expiry = datetime.now() + timedelta(
                    seconds=json["expires_in"] - 10
                )
                return_value = True
        except KeyError as exception:
            _LOGGER.error(f"Error processing access token response: {exception}")
            await self.__handle_response_error(response, SessionError)

        return return_value

    async def __handle_response_error(self, response, error_class):
        status = response.status
        _LOGGER.error(f"Unexpected response when creating session ({status})")
        response = await response.text()
        _LOGGER.error(response)

        raise error_class(status, response)



from dataclasses import dataclass
import json
from typing import Any, Optional

@dataclass
class ScheduleStatus:
    is_active: bool = False

@dataclass
class Schedule:
    start_day: int
    start_time: str
    end_day: int
    end_time: str
    status: ScheduleStatus
    uid: str = None # Optional uid value - when creating new schedules uid is not required and is easier to ommit

    @property
    def is_active(self):
        return self.status.is_active

    @property
    def dict(self):
        d = {
            "start_day": self.start_day,
            "start_time": self.start_time,
            "end_day": self.end_day,
            "end_time": self.end_time,
            "status": {
                "is_active": self.status.is_active
            }
        }

        # If uid is set, add it to the start
        if self.uid:
          ud = {"uid": self.uid}
          ud.update(d)
          d = ud

        return d

    def to_json(self):
        return json.dumps(self.dict, ensure_ascii=False)



from distutils.log import error
from os import access
from typing import Any, Dict
import aiohttp
import async_timeout
import asyncio
import logging
import socket
from datetime import datetime
import re

from ..errors import APIError

TIMEOUT=10
HEADERS = {"Content-type": "application/json; charset=UTF-8"}

_LOGGER: logging.Logger = logging.getLogger(__package__)

class Helpers:
    def auth_headers(self, access_token: str) -> Dict[str, str]:
        auth_header = {"Authorization": f"Bearer {access_token}"}
        combined_headers = HEADERS.copy()
        combined_headers.update(auth_header)
        return combined_headers

    def lazy_convert_to_datetime(self, date_string: str) -> datetime:
        if date_string is None or type(date_string) is not str:
            return None
        
        # Convert a 'Z' ending string to +00:00 for correct support
        date_string = re.sub(r"Z$", "+00:00", date_string)

        dt = None
        # Example: 2022-01-25T09:00:00+00:00
        try:
            dt = datetime.fromisoformat(date_string)
        except ValueError as e:
            _LOGGER.warning("Tried to convert '%s' to datetime but got: %s", date_string, e)
        
        return dt

    def lazy_iso_format_datetime(self, date_time: datetime) -> str:
        if type(date_time) is not datetime:
            return None

        return date_time.isoformat()

    def default_serialisation(self, obj):
        if hasattr(obj, 'to_json'):
            return obj.to_json()
        raise TypeError(f'Object of type {obj.__class__.__name__} is not JSON serializable')
            

class APIWrapper:
    def __init__(self, session: aiohttp.ClientSession, timeout: int = TIMEOUT) -> None:
        self._timeout: int = timeout
        self._session: aiohttp.ClientSession = session

    async def get(self, url: str, headers: Dict[str, Any], params: Dict[str, Any] = None, exception_class=APIError) -> aiohttp.ClientResponse:
        return await self.__wrapper(method="get", url=url, params=params, headers=headers, exception_class=exception_class)

    async def put(self, url: str, body: Any, headers: Dict[str, Any], params: Dict[str, Any] = None, exception_class=APIError) -> aiohttp.ClientResponse:
        return await self.__wrapper(method="put", url=url, params=params, data=body, headers=headers, exception_class=exception_class)
    
    async def post(self, url: str, body: Any, headers: Dict[str, Any], params: Dict[str, Any] = None, exception_class=APIError) -> aiohttp.ClientResponse:
        return await self.__wrapper(method="post", url=url, params=params, data=body, headers=headers, exception_class=exception_class)

    async def __wrapper(
        self,
        method: str,
        url: str,
        data: Dict[str, Any] = {},
        headers: Dict[str, Any] = {},
        params: Dict[str, Any] = {},
        exception_class=APIError
    ) -> aiohttp.ClientResponse:
        """Get information from the API."""
        try:
            async with async_timeout.timeout(self._timeout):
                response = None

                if method == "get":
                    response = await self._session.get(url, headers=headers, params=params)

                elif method == "put":
                    response = await self._session.put(url, headers=headers, json=data)

                elif method == "post":
                    response = await self._session.post(url, headers=headers, json=data)
                else:
                    raise ValueError("Method '%s' not supported", method)

                if response is None:
                    raise APIError("Unexpected error from Pod Point API. Received a None response when querying.")

                if response.status < 200 or response.status > 202:
                    self.__handle_response_error(response=response, exception_class=exception_class)

                return response

        except asyncio.TimeoutError as exception:
            _LOGGER.error(
                "Timeout error fetching information from %s - %s",
                url,
                exception,
            )
            raise exception

        except (KeyError, TypeError) as exception:
            _LOGGER.error(
                "Error parsing information from %s - %s",
                url,
                exception,
            )
            raise exception

        except (aiohttp.ClientError, socket.gaierror) as exception:
            _LOGGER.error(
                "Error fetching information from %s - %s",
                url,
                exception,
            )
            self.__handle_response_error(response=response, exception_class=exception_class)

        except Exception as exception:  # pylint: disable=broad-except
            _LOGGER.error("Something really wrong happened")
            raise exception

    async def __handle_response_error(self, response: aiohttp.ClientResponse, exception_class):
        status = response.status
        _LOGGER.error(f"Unexpected response when creating session ({status})")
        response = await response.text()
        _LOGGER.error(response)

        raise exception_class(status, response) 


class APIError(Exception):
    pass

class AuthError(APIError):
    def __init__(self, status, response):
        message = f'Auth Error ({status}) - {response}'
        super().__init__(message)

class SessionError(APIError):
    def __init__(self, status, response):
        message = f'Session Error ({status}) - {response}'
        super().__init__(message)



from typing import Dict, Any, List
from .pod import Pod
from .schedule import Schedule, ScheduleStatus
from .charge import Charge

class PodFactory:
    def build_pods(self, pods_response: Dict[str, Any]) -> List[Pod]:
        pods = []

        pods_data = pods_response.get('pods', None) 
        if pods_data is None:
            return pods

        for pod_data in pods_data:
            pods.append(Pod(data=pod_data))

        return pods

class ScheduleFactory:
    def build_schedules(self, enabled: bool, start_time: str = "00:00:00", end_time: str = "00:00:01") -> List[Schedule]:
        schedules = []

        for i in range(7):
            day = i + 1

            schedule = Schedule(
                start_day=day,
                start_time=start_time,
                end_day=day,
                end_time=end_time,
                status=ScheduleStatus(is_active=enabled)
            )

            schedules.append(schedule)

        return schedules


class ChargeFactory:
    def build_charges(self, charge_response: Dict[str, Any]) -> List[Charge]:
        charges = []

        charge_data = charge_response.get('charges', None) 
        if charge_data is None:
            return charges

        for charge in charge_data:
            charges.append(Charge(data=charge))

        return charges



from setuptools import setup, find_packages
from podpointclient.version import __version__ as version

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="podpointclient",
    version=version,
    author="Matthew Rayner",
    author_email="hello@rayner.io",
    description="A simple API client for Pod Point (https://pod-point.com) aimed at home users",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mattrayner/podpointclient",
    project_urls={
        "Bug Tracker": "https://github.com/mattrayner/podpointclient/issues",
    },
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(exclude=['tests']),
    python_requires=">=3.7",
    keywords='Pod Point PodPoint',
    include_package_data=True,
)


from dataclasses import dataclass
from datetime import datetime
from sqlite3 import DatabaseError
from typing import Dict, Any
from .schedule import Schedule, ScheduleStatus
from enum import auto
from strenum import StrEnum, KebabCaseStrEnum
from .helpers.helpers import Helpers
import json


class StatusName(StrEnum):
    AVAILABLE      = "Available"
    UNAVAILABLE    = "Unavailable"
    CHARGING       = "Charging"
    OUT_OF_SERVICE = "Out of Service"


class StatusKeyName(KebabCaseStrEnum):
    AVAILABLE      = auto()
    UNAVAILABLE    = auto()
    CHARGING       = auto()
    OUT_OF_SERVICE = auto()


@dataclass
class Socket:
    type: str
    description: str
    ocpp_name: str
    ocpp_code: int

    @property
    def dict(self):
        return {
            "type": self.type,
            "description": self.description,
            "ocpp_name": self.ocpp_name,
            "ocpp_code": self.ocpp_code
        }

    def to_json(self):
        return json.dumps(self.dict, ensure_ascii=False)


class Pod:
    def __init__(self, data: Dict[str, Any]):
        helpers = Helpers()

        self.id: int                   = data.get('id', None)
        self.name: str                 = data.get('name', None)
        self.ppid: str                 = data.get('ppid', None)
        self.payg: bool                = data.get('payg', None)
        self.home: bool                = data.get('home', None)
        self.public: bool              = data.get('public', None)
        self.ev_zone: bool              = data.get('evZone', None)
        self.address_id: int           = data.get('address_id', None)
        self.description: str          = data.get('description', "")
        self.commissioned_at: datetime = helpers.lazy_convert_to_datetime(data.get('commissioned_at', None))
        self.created_at: datetime      = helpers.lazy_convert_to_datetime(data.get('created_at', None))
        self.last_contact_at: datetime = helpers.lazy_convert_to_datetime(data.get('last_contact_at', None))
        self.contactless_enabled: bool = data.get('contactless_enabled', None)
        self.unit_id: int              = data.get('unit_id', None)
        self.timezone: str             = data.get('timezone', None)
        self.price: int                = data.get('price', None)

        model_data = data.get('model', {})
        self.model = self.Model(
            id                   = model_data.get('id', None),
            name                 = model_data.get('name', None),
            vendor               = model_data.get('vendor', None),
            supports_payg        = model_data.get('supports_payg', False),
            supports_ocpp        = model_data.get('supports_ocpp', False),
            supports_contactless = model_data.get('supports_contactless', False),
            image_url            = model_data.get('image_url', None)
        )

        location_data = data.get('location', {})
        self.location = self.Location(
            lat = location_data.get('lat', 0.0),
            lng = location_data.get('lng', 0.0)
        )

        self.statuses = []
        statuses_data = data.get('statuses', [])
        for status in statuses_data:
            self.statuses.append(
                self.Status(
                    id       = status.get('id', None),
                    name     = status.get('name', None),
                    key_name = status.get('key_name', None),
                    label    = status.get('label', None),
                    door     = status.get('door', None),
                    door_id  = status.get('door_id', None),
                )
            )

        self.unit_connectors = []
        unit_connectors_data = data.get('unit_connectors', [])
        for unit_connector in unit_connectors_data:
            connector_data = unit_connector.get('connector', {})

            socket_obj = None
            socket_data = connector_data.get('socket', None)
            if socket_data is not None:
                socket_obj = Socket(
                    type = socket_data.get('type', None),
                    description = socket_data.get('description', None),
                    ocpp_name = socket_data.get('ocpp_name', None),
                    ocpp_code = socket_data.get('ocpp_code', None),
                )

            self.unit_connectors.append(
                self.Connector(
                    id = connector_data.get('id', None),
                    door = connector_data.get('door', None),
                    door_id = connector_data.get('door_id', None),
                    power = connector_data.get('power', None),
                    current = connector_data.get('current', None),
                    voltage = connector_data.get('voltage', None),
                    charge_method = connector_data.get('charge_method', None),
                    has_cable = connector_data.get('has_cable', None),
                    socket=socket_obj
                )
            )

        self.charge_schedules = []
        charge_schedules_data = data.get('charge_schedules', [])
        for charge_schedule_data in charge_schedules_data:
            status_data = charge_schedule_data.get('status', None)
            status_obj = None
            if status_data:
                status_obj = ScheduleStatus(
                    is_active = status_data.get('is_active', None)
                )

            self.charge_schedules.append(
                Schedule(
                    uid = charge_schedule_data.get('uid', None),
                    start_day = charge_schedule_data.get('start_day', None),
                    start_time = charge_schedule_data.get('start_time', None),
                    end_day = charge_schedule_data.get('end_day', None),
                    end_time = charge_schedule_data.get('end_time', None),
                    status = status_obj
                )
            )
    
    @property
    def dict(self) -> Dict[str, Any]:
        helpers = Helpers()
        
        d = {
            "id": self.id,
            "name": self.name,
            "ppid": self.ppid,
            "payg": self.payg,
            "home": self.home,
            "public": self.public,
            "evZone": self.ev_zone,
            "location": self.location.dict,
            "address_id": self.address_id,
            "description": self.description,
            "commissioned_at": helpers.lazy_iso_format_datetime(self.commissioned_at),
            "created_at": helpers.lazy_iso_format_datetime(self.created_at),
            "last_contact_at": helpers.lazy_iso_format_datetime(self.last_contact_at),
            "contactless_enabled": self.contactless_enabled,
            "unit_id": self.unit_id,
            "timezone": self.timezone,
            "model": self.model.dict,
            "price": self.price,
            "statuses": [],
            "unit_connectors": [],
            "charge_schedules": []
        }

        for status in self.statuses:
            d['statuses'].append(status.dict)

        for unit_connector in self.unit_connectors:
            d['unit_connectors'].append(
                { "connector": unit_connector.dict }
            )

        for charge_schedule in self.charge_schedules:
            d['charge_schedules'].append(charge_schedule.dict)

        return d
    
    def to_json(self):
        return json.dumps(self.dict, ensure_ascii=False)


    @dataclass
    class Model:
        id: int
        name: str
        vendor: str
        supports_payg: bool = False
        supports_ocpp: bool = False
        supports_contactless: bool = False
        image_url: str = None

        @property
        def model(self):
            return self.name

        @property
        def dict(self):
            return {
                "id": self.id,
                "name": self.name,
                "vendor": self.vendor,
                "supports_payg": self.supports_payg,
                "supports_ocpp": self.supports_ocpp,
                "supports_contactless": self.supports_contactless,
                "image_url": self.image_url
            }

        def to_json(self):
            return json.dumps(self.dict, ensure_ascii=False)


    @dataclass
    class Location:
        lat: float
        lng: float

        @property
        def dict(self):
            return {
                "lat": self.lat,
                "lng": self.lng
            }

        def to_json(self):
            return json.dumps(self.dict, ensure_ascii=False)

    
    @dataclass
    class Status:
        id: int
        name: StatusName
        key_name: StatusKeyName
        label: StatusName
        door: str
        door_id: int

        @property
        def dict(self):
            return {
                "id": self.id,
                "name": self.name,
                "key_name": self.key_name,
                "label": self.label,
                "door": self.door,
                "door_id": self.door_id
            }

        def to_json(self):
            return json.dumps(self.dict, ensure_ascii=False)

    
    @dataclass
    class Connector:
        id: int
        door: str
        door_id: int
        power: int
        current: int
        voltage: int
        charge_method: str
        has_cable: bool
        socket: Socket

        @property
        def dict(self):
            return {
                "id": self.id,
                "door": self.door,
                "door_id": self.door_id,
                "power": self.power,
                "current": self.current,
                "voltage": self.voltage,
                "charge_method": self.charge_method,
                "has_cable": self.has_cable,
                "socket": self.socket.dict
            }

        def to_json(self):
            return json.dumps(self.dict, ensure_ascii=False)
